(function (parent) {

  function formatPart(value) {
    var remain = (value % 10);
    var divided = (value - remain) / 10;
    return (divided).toString() + (remain).toString();
  }

  function formatResult(desiredDateTime, format) {
    if (!format)
      return desiredDateTime;

    var formattedResult = desiredDateTime.year + '/' + formatPart(desiredDateTime.month) + '/' + formatPart(desiredDateTime.day);

    if ((format.indexOf('HH') > -1) && (desiredDateTime.hours))
      formattedResult += ' ' + formatPart(desiredDateTime.hours);

    if ((format.indexOf('mm') > -1) && (desiredDateTime.minutes))
      formattedResult += ':' + formatPart(desiredDateTime.minutes);

    if ((format.indexOf('ss') > -1) && (desiredDateTime.seconds))
      formattedResult += ':' + formatPart(desiredDateTime.seconds);

    return formattedResult;
  }

  function div(a, b) {
    return parseInt((a / b));
  }

  function gregorian_to_jalali(gy, gm, gd) {
    g_d_m = [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334];
    if (gy > 1600) {
      jy = 979;
      gy -= 1600;
    } else {
      jy = 0;
      gy -= 621;
    }
    gy2 = (gm > 2) ? (gy + 1) : gy;
    days = (365 * gy) + (parseInt((gy2 + 3) / 4)) - (parseInt((gy2 + 99) / 100)) + (parseInt((gy2 + 399) / 400)) - 80 + gd + g_d_m[gm - 1];
    jy += 33 * (parseInt(days / 12053));
    days %= 12053;
    jy += 4 * (parseInt(days / 1461));
    days %= 1461;
    if (days > 365) {
      jy += parseInt((days - 1) / 365);
      days = (days - 1) % 365;
    }
    jm = (days < 186) ? 1 + parseInt(days / 31) : 7 + parseInt((days - 186) / 30);
    jd = 1 + ((days < 186) ? (days % 31) : ((days - 186) % 30));

    return {
      year: jy,
      month: jm,
      day: jd
    }
  }

  function jalali_to_gregorian(jy, jm, jd) {
    if (jy > 979) {
      gy = 1600;
      jy -= 979;
    } else {
      gy = 621;
    }
    days = (365 * jy) + ((parseInt(jy / 33)) * 8) + (parseInt(((jy % 33) + 3) / 4)) + 78 + jd + ((jm < 7) ? (jm - 1) * 31 : ((jm - 7) * 30) + 186);
    gy += 400 * (parseInt(days / 146097));
    days %= 146097;
    if (days > 36524) {
      gy += 100 * (parseInt(--days / 36524));
      days %= 36524;
      if (days >= 365) days++;
    }
    gy += 4 * (parseInt(days / 1461));
    days %= 1461;
    if (days > 365) {
      gy += parseInt((days - 1) / 365);
      days = (days - 1) % 365;
    }
    gd = days + 1;
    sal_a = [0, 31, ((gy % 4 == 0 && gy % 100 != 0) || (gy % 400 == 0)) ? 29 : 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
    for (gm = 0; gm < 13; gm++) {
      v = sal_a[gm];
      if (gd <= v) break;
      gd -= v;
    }

    return {
      year: gy,
      month: gm,
      day: gd
    }
  }

  function get_year_month_day(date) {
    var convertDate;
    var y = date.substr(0, 4);
    var m = date.substr(5, 2);
    var d = date.substr(8, 2);
    convertDate = gregorian_to_jalali(y, m, d);
    return convertDate;
  }

  function get_hour_minute_second(time) {
    var convertTime = [];
    convertTime[0] = time.substr(0, 2);
    convertTime[1] = time.substr(3, 2);
    convertTime[2] = time.substr(6, 2);
    return convertTime;
  }

  function convertDate(date) {
    var convertDateTime = get_year_month_day(date.substr(0, 10));
    convertDateTime = convertDateTime[0] + "/" + convertDateTime[1] + "/" + convertDateTime[2] + " " + date.substr(10);
    return convertDateTime;
  }

  function GetPersianMonthCaption(month) {
    switch (month) {
      case 1: return "فروردین"; break;
      case 2: return "اردیبهشت"; break;
      case 3: return "خرداد"; break;
      case 4: return "تیر"; break;
      case 5: return "مرداد"; break;
      case 6: return "شهریور"; break;
      case 7: return "مهر"; break;
      case 8: return "آبان"; break;
      case 9: return "آذر"; break;
      case 10: return "دی"; break;
      case 11: return "بهمن"; break;
      case 12: return "اسفند"; break;
    }
  }

  function GregorianDateToJalaliDate(jsTicks, format) {

    var date = new Date(jsTicks);
    var persianDate = gregorian_to_jalali(date.getFullYear(), date.getMonth() + 1, date.getDate());
    persianDate.hours = date.getHours();
    persianDate.minutes = date.getMinutes();
    persianDate.seconds = date.getSeconds();

    return formatResult(persianDate, format);
  }

  function JalaliDateToGregorianDateSplitted(jyear, jmonth, jday, hours, minutes, seconds, format) {
    var gregorianDate = jalali_to_gregorian(jyear, jmonth, jday);
    gregorianDate.hours = hours;
    gregorianDate.minutes = minutes;
    gregorianDate.seconds = seconds;

    return formatResult(gregorianDate, format);
  }

  function JalaliDateToGregorianDate(jalaliDate, format) {

    var splittedDateTime = jalaliDate.split(' ');
    var datePart = (splittedDateTime.length > 0) ? splittedDateTime[0] : "";
    var timePart = (splittedDateTime.length > 1) ? splittedDateTime[1] : "";

    var splittedTime = timePart.split(':');
    var hours = parseInt((splittedTime.length > 0) ? splittedTime[0] : "0");
    var minutes = parseInt((splittedTime.length > 1) ? splittedTime[1] : "0");
    var seconds = parseInt((splittedTime.length > 2) ? splittedTime[2] : "0");

    var splittedDate = datePart.split('/');
    var jyear = parseInt((splittedDate.length > 0) ? splittedDate[0] : "0");
    var jmonth = parseInt((splittedDate.length > 1) ? splittedDate[1] : "0");
    var jday = parseInt((splittedDate.length > 2) ? splittedDate[2] : "0");

    return JalaliDateToGregorianDateSplitted(jyear, jmonth, jday, hours, minutes, seconds, format);
  }

  function DotnetTicksToJavascriptMilliseconds(input) {
    var epochTicks = 621355968000000000,    // the number of .net ticks at the unix epoch
      ticksPerMillisecond = 10000,        // there are 10000 .net ticks per millisecond
      jsTicks = 0,                        // ticks in javascript environment
      jsDate;                             // Date in javascript environment

    jsTicks = (input - epochTicks) / ticksPerMillisecond;

    var utcOffset = new Date().getTimezoneOffset() * 60 * 1000;
    jsTicks = jsTicks + utcOffset;

    return jsTicks;
  }

  function JavascriptMillisecondsToDotnetTicks(input) {
    var epochTicks = 621355968000000000,    // the number of .net ticks at the unix epoch
      ticksPerMillisecond = 10000,        // there are 10000 .net ticks per millisecond
      netTicks = 0;                       // ticks in javascript environment

    var utcOffset = new Date().getTimezoneOffset() * 60 * 1000;
    input = input - utcOffset;

    netTicks = (input * ticksPerMillisecond) + epochTicks;
    return netTicks;
  }

  function dotnetTicks2GregorianDate(dotnetTicks, format) {
    var jsTicks = DotnetTicksToJavascriptMilliseconds(dotnetTicks);
    return new Date(jsTicks, format);
  }

  function DotnetTicksToJalaliDate(dotnetTicks, format) {
    var jsTicks = DotnetTicksToJavascriptMilliseconds(dotnetTicks);
    return GregorianDateToJalaliDate(jsTicks, format);
  }

  this.jsDateConvetor = {
    GetPersianMonthCaption: GetPersianMonthCaption,
    GregorianDateToJalaliDate: GregorianDateToJalaliDate,
    JalaliDateToGregorianDate: JalaliDateToGregorianDate,
    JalaliDateToGregorianDateSplitted: JalaliDateToGregorianDateSplitted,
    JavascriptMillisecondsToDotnetTicks: JavascriptMillisecondsToDotnetTicks,
    DotnetTicksToJavascriptMilliseconds: DotnetTicksToJavascriptMilliseconds,
    DotnetTicksToJalaliDate: DotnetTicksToJalaliDate,
  }
}(window));
